# compute the times of action(rec|click|msg) for each user
# compute the click, msg as positive vote, the rec as false vote, then use "wilson interval"
# @author: cheng yan
# @date: 2012-08-31

from math import sqrt

def getActionScore(action):  
  if action == "rec": 
    return 0
  elif action == "click"  :    
    return 1 
  else:  
    return 2

def compute_interaction(data): 
  interaction = {}

  for line in data:   
    (userA,userB,times,action) = line.split(' ')  
    action = action[:-1]
    key = userB + " " + action
    interaction.setdefault(key, 0)
    interaction[key] += 1

  return interaction 

def compute_user_history_interaction(trainFile):
  records = []
  lineList = []
  lineNum = 1
  result = []
  
  lineList = [line for line in file(trainFile)]
  for line in lineList:
    if lineNum == 1:	#ignore the title in first line
      lineNum += 1
      continue

    records.append(line)
    lineNum += 1

  interaction = compute_interaction(records)  

  out = file('user_interaction.txt', 'w')
  for (key, times) in interaction.items(): 
    out.write('%s %d' % (key, times))
    out.write('\n')

  for (key, times) in interaction.items():
    user, action = key.split(' ');
    result.append((user, action, times))
  
  return result

#get the weight for each type of action
def get_action_weight(action):
  pop = 0;
  if action == "rec":
    pop = 1
  elif action == "click":
    pop = 1.5
  elif action == "msg":
    pop = 2

  return pop;

def compute_confidence(ups, downs): 
  n = ups + downs 
  if n == 0: 
    return 0
  z = 1.0 #1.0 = 85%, 1.6 = 95% 
  phat = float(ups) / n 

  return (phat+z*z/(2*n)-z*sqrt((phat*(1-phat)+z*z/(4*n))/n))/(1+z*z/n) 

#trainFile line like: [userA, userB, action_times, action_type(rec|click|msg)]
def compute_user_popularity(trainFile, user_popularity_file):
  ups = {}
  downs = {}
  popDict = {}
  rankedscores = []
  result = []
  
  print "-----compute_user_history_interaction ... "
  
  interaction = compute_user_history_interaction(trainFile)
  
  print "-----compute_user_popularity ... "
  
  # compute click, msg as positive vote(ups), compute rec as false vote(downs)
  for (user, action, times) in interaction[0:len(interaction)]:
    ups.setdefault(user, 0)
    downs.setdefault(user, 0)
    ups.setdefault(user, 0)
    downs.setdefault(user, 0)
    if action != "rec":
      ups[user] += get_action_weight(action) * times
    else:
      downs[user] += get_action_weight(action) * times

  # compute confidence interval using wilson interval
  for (user, votes) in ups.items():
    popDict[user] = compute_confidence(ups[user], downs[user])
  
  ranked_popularity = [(popularity, user) for (user, popularity) in popDict.items()]
  ranked_popularity.sort()
  ranked_popularity.reverse()

  print "-----ranking_user_popularity ... "
  
  result = [(user, popularity) for (popularity, user) in ranked_popularity[0:len(ranked_popularity)]]

  print "-----output user_popularity ... "
  
  out = file(user_popularity_file, 'w')
  for (user, popularity) in result[0:len(result)]: 
    out.write('%s %f\n' % (user, popularity))

  print "-----Ending ... "

  return result